import java.util.Hashtable;
import java.util.Enumeration;
import com.ibm.xml.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import javax.swing.*;
import ui.DOMTree;

/** SimpleTreeView2 builds on SimpleTreeView
 *  to show how store the errors and print them after parsing.
 */
public class SimpleTreeView2 extends JFrame {
    
    DOMParser parser;
    Errors errors;

    public static void main (String args[]) {

        if (args.length > 0) {
            String filename = args[0];

            try {
                SimpleTreeView2 frame = new SimpleTreeView2(filename) ;
                frame.addWindowListener(new java.awt.event.WindowAdapter() {
                 public void windowClosing(java.awt.event.WindowEvent e) {
                     System.exit(0);
                 }
                });
                frame.setSize(300, 400);
                frame.setVisible(true);
                } catch (Exception e) {
                    e.printStackTrace(System.err);
                }
        }
    }

    public SimpleTreeView2 (String filename) {
        super("SimpleTreeView2: "+filename);
        try {
            parser = new DOMParser();
            errors = new Errors();
            parser.setNodeExpansion(DOMParser.FULL);
            parser.setErrorHandler(errors);
            parser.parse(filename);
            Document document = parser.getDocument();
            Hashtable errorNodes = errors.getErrorNodes();
            Enumeration elements = errorNodes.elements();
            while (elements.hasMoreElements()) {
                System.out.println( (String)elements.nextElement());
            }
            getContentPane().add(new JScrollPane(new DOMTree(document)));
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
    }

    class Errors implements ErrorHandler {
        
        Hashtable errorNodes = new Hashtable();
     
        public void warning(SAXParseException ex) {
            store(ex, "[Warning]");
        }

        public void error(SAXParseException ex) {
            store(ex, "[Error]");
        }

        public void fatalError(SAXParseException ex) throws SAXException {
            store(ex, "[Fatal Error]");
        }
        
        public Hashtable getErrorNodes() {
            return errorNodes;
        }

        public Object getError(Node node) {
            return errorNodes.get(node);
        }
        
        public void resetErrors() {
            errorNodes.clear();
        }
        
        void store(SAXParseException ex, String type) {

            // build error text
            String errorString= type+" at line number, "+ex.getLineNumber()
                +": "+ex.getMessage()+"\n";

            Node currentNode = parser.getCurrentNode();
            if (currentNode == null) return;

            // accumulate any multiple errors per node in the Hashtable.
            String previous = (String) errorNodes.get(currentNode);
            if (previous != null)
                errorNodes.put(currentNode, previous +errorString);
            else
                errorNodes.put(currentNode, errorString);
        }   
    }   
}